@@ -3,7 +3,7 @@ context("radixorder, GRP, qF, qG")
 # print(str(wlddev))
 # rm(list = ls())
 
-if(!is.null(attributes(identical(FALSE, TRUE)))) stop("OECD label issue")
+
 
 NCRAN <- Sys.getenv("NCRAN") == "TRUE"
 set.seed(101)
@@ -147,7 +147,7 @@ test_that("GRP gives errors for wrong input", {
   expect_error(GRP(1:10, 1))
   expect_error(GRP(1:10, ~ cyl))
   expect_error(GRP(1:10, "cyl"))
-  expect_error(GRP(mtcars, TRUE))
+  # expect_error(GRP(mtcars, TRUE))
   expect_error(GRP(mtcars, ~ cyl + bla))
   expect_error(GRP(mtcars, c("bal","cyl")))
   expect_error(GRP(mtcars, 11:12))
